<?php
/**
 * Class handling Albireo install packages
 * @package    Albireo I
 * @subpackage Core
 * @copyright  Copyright (c) 2009 Albireo Solutions. All rights reserved
 * @license    GNU/AGPL http://www.gnu.org/licenses/agpl.html
 *
 * Albireo is free software! You can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *	GNU Affero General Public License for more details.
 */
class PackageManager {

    private $package_name;
    private $destruct = true;
    private $issue = array (
    		"errors" => 0,
    		"installed" => 0,
    		"ok messages" => "",
    		"error messages" => "" );

    /**
     * Class handling Albireo install packages
     */
    public function __construct ()
    {
    } // end __construct

    /**
     * removes all temporary files and destructs this object instance
     */
    public function __destruct ()
    {
    	$this -> removeAllTemporary ();
    } // end __destruct

    /**
     * forbids removing temporary files unless we call removeAllTemporary method
     */
    public function forbidDestruct () {
        $this -> destruct = false;
    }

    /**
     * downloads package from FTP server
     * @param String $url target source url (e.g "ftp.fi.muni.cz")
     * @param String $package_name package name
     * @return array associative array
     * "errors" => number of files we couln't install,
     * "installed" => number of successfully installed files,
     * "ok messages" => ok messages,
     * "error messages" => error messages
     */
    public function download ( $url, $package_name = "none" )
    {

	$this -> package_name = ( $package_name == "none" ) ? substr ( strrchr ( $url, "/" ), 1 ) : $package_name;

	// manage existence of folder for temporary files ////////////////////////////
	if ( !file_exists ( "temporary" ) ) {
	    if ( !@mkdir ( "temporary" ) ) {
	    	$this -> issue [ "errors" ] = $this -> issue [ "errors" ] + 1;
	    	$this -> issue [ "error messages" ] = '<p><strong class="warning">'. _( "Fatal Error - directory temporary cannot be created!" ) .'</strong> - write: '. substr ( sprintf ( '%o', fileperms ( '.' ) ), -4 ) .'</p>';
	    	return $this -> issue;
	    }
    	}
	chdir ( "temporary" );
	//////////////////////////////////////////////////////////////////////////////

	$ftp_server = 'ftp.fi.muni.cz';
	// set up basic connection
	$conn_id = @ftp_connect($ftp_server);
	$login_result = @ftp_login($conn_id, 'anonymous', 'guest');

	if ((!$conn_id) || (!$login_result)) {
	    $this -> issue [ "errors" ] = $this -> issue [ "errors" ] + 1;
	    $this -> issue [ "error messages" ] = '<p><strong class="warning">'. _( "Fatal Error - FTP not connected!" ) .'</strong> - write: '. substr ( sprintf ( '%o', fileperms ( '.' ) ), -4 ) .'</p>';
	    return $this -> issue;
	}

	@ftp_chdir($conn_id, "pub");
	@ftp_chdir($conn_id, "local");
	@ftp_chdir($conn_id, "albireo");

	@ftp_pasv($conn_id, true);
	set_time_limit(9000);
	if (!@ftp_get($conn_id, $this -> package_name, $this -> package_name, FTP_BINARY)) {
	    $this -> issue [ "errors" ] = $this -> issue [ "errors" ] + 1;
	    $this -> issue [ "error messages" ] = '<p><strong class="warning">'. _( "Package was not downloaded" ) .'</strong></p>';
	    return $this -> issue;
	}

	// close the connection
	@ftp_close($conn_id);
    	chdir ( ".." );

    	return $this -> issue;

    } // end download

    /**
     * extracts Albireo package file using php zip extension.
     * Unfortunatelly some servers have problem with this operation if the zip file is larger
     * @param String $package_name package name
     * @return array associative array of overall package manager results
     * "errors" => number of files we couln't install,
     * "installed" => number of successfully installed files,
     * "ok messages" => ok messages,
     * "error messages" => error messages
     */
    public function extractZip ( $package_name )
    {

        if ( !file_exists ( $package_name ) ) {
            $this -> issue [ "errors" ] = $this -> issue [ "errors" ] + 1;
            $this -> issue [ "error messages" ] = '<p><strong class="warning">'. _( "Fatal Error - zip package can not be found!" ) .'</strong></p>';
            return $this -> issue;
        }
    	$zip = new ZipArchive;
        if ( @$zip -> open ( $package_name ) === true ) {
            $zip -> extractTo ( "." );
            $zip -> close ();
        } else {
            $this -> issue [ "errors" ] = $this -> issue [ "errors" ] + 1;
            $this -> issue [ "error messages" ] = '<p><strong class="warning">'. _( "Fatal Error - downloaded zip package cannot be opened!" ) .'</strong></p>';
        }
        return $this -> issue;

    } // end extractZip

    /**
     * installs Albireo package. Unzips it and move all objects to Albireo folder
     * @param <type> $package_name name of package previous downloaded to folder temporary
     * @return array associative array of overall package manager results
     * "errors" => number of files we couln't install,
     * "installed" => number of successfully installed files,
     * "ok messages" => ok messages,
     * "error messages" => error messages
     */
    public function installPackage ( $package_name = "none" )
    {

    	chdir ( "temporary" );

        $this -> package_name = ( $package_name == "none" ) ? $this -> package_name : $package_name;

	if ( !file_exists ( "./package/" ) && substr ( strrchr ( $this -> package_name, "." ), 1 ) == "zip" ) {
	    $begin_errors = $this -> issue [ "errors" ];
	    $this -> extractZip ( $this -> package_name ); // extract ziped package
        if ( $this -> issue [ "errors" ] > $begin_errors ) {
            return $this -> issue;
        }
	}

	chdir ( ".." );

	if ( $this -> issue [ "errors" ] > 0 ) {
	    return $this -> issue;
	}

	$this -> issue [ "ok messages" ] .= '<ul class="green">';
	$this -> issue [ "error messages" ] .= '<ul class="warning">';

	if ( file_exists ( "./temporary/package/" ) ) {
	    $this -> installDir ( "./temporary/package/", "./" );
	} else {
	    unlink ( "./temporary/". $this -> package_name );
	    $this -> installDir ( "./temporary/", "./" );
	}

	$this -> issue [ "ok messages" ] .= '</ul>';
	$this -> issue [ "error messages" ] .= '</ul>';

	return $this -> issue;


    } // end install

    private function installDir ( $package_directory, $install_directory )
    {

    	$folder = opendir ( $package_directory );

    	while ( $file = readdir ( $folder ) ) {

	    if ( is_dir ( $package_directory . $file ) ) {

	    	// installation in case of founding subfolder - recursion ////////////////////
	    	if ( $file != "." && $file != ".."  ) {
	    	    if ( !file_exists ( $install_directory . $file ) ) {
	    		if ( !@mkdir ( $install_directory . $file ) ) {
	    			$this -> issue [ "errors" ] = $this -> issue [ "errors" ] + 1;
				$this -> issue [ "error messages" ] .= '<li>'. _( "error - folder does not exist and cannot be created:" ). ' '. $install_directory . $file .'</li>';
	    		} else {
	    		    $this -> installDir ( $package_directory ."$file/", $install_directory ."$file/" );
	    		}
		    } else {
		    	$this -> installDir ( $package_directory ."$file/", $install_directory ."$file/" );
		    }
	    	}
	    	//////////////////////////////////////////////////////////////////////////////

	    } else {
	    	// make copy of file - install it ////////////////////////////////////////////
	    	if ( @copy ( $package_directory . $file, $install_directory . $file ) ) {
	    	    $this -> issue [ "installed" ] = $this -> issue [ "installed" ] + 1;
		    $this -> issue [ "ok messages" ] .= '<li>'. _( "file successfully installed:" ). ' '. $install_directory . $file ."</li>\n";
	    	} else {
	    	    $this -> issue [ "errors" ] = $this -> issue [ "errors" ] + 1;
	    	    $this -> issue [ "error messages" ] .= '<li>'._( "error - file cannot be installed:" ). ' '. $install_directory . $file ."</li>\n";
	    	}
	    	//////////////////////////////////////////////////////////////////////////////

	    }

	}

	closedir ( $folder );

    } // end installDir

    /**
     * removes all temporary files
     */
    public function removeAllTemporary ()
    {
    	if ( file_exists ( "temporary" ) ) {
    	    chdir ( "temporary" );
    	    $this -> removeDir ( "./" );
    	    chdir ( ".." );
    	}

    } // end removeAllTemporary

    private function removeDir ( $directory )
    {
    	$folder = opendir ( $directory );

    	while ( $file = readdir ( $folder ) ) {

	    if ( is_dir ( $directory . $file ) ) {

	    	// deleting in case of founding subfolder - recursion ////////////////////////
	    	if ( $file != "." && $file != ".."  ) {
		    $this -> removeDir ( $directory ."$file/" );
		    if ( !( @rmdir ( $directory ."$file/" ) ) ) {
		    	$this -> issue [ "errors" ] = $this -> issue [ "errors" ] + 1;
	    	    	$this -> issue [ "error messages" ] .= '<li>'._( "error - folder cannot be deleted:" ). ' '. $directory . $file .', '. _( "write:" ) .' '. substr ( sprintf ( '%o', fileperms ( $directory . $file ) ), -4 ) ."</li>\n";
		    }
	    	}
	    	//////////////////////////////////////////////////////////////////////////////

	    } else {

	    	// delete temporary file /////////////////////////////////////////////////////
	    	if ( !( @unlink ( $directory . $file ) ) ) {
	    	    $this -> issue [ "errors" ] = $this -> issue [ "errors" ] + 1;
	    	    $this -> issue [ "error messages" ] .= '<li>'._( "error - file cannot be deleted:" ). ' '. $directory . $file .', '. _( "write:" ) .' '. substr ( sprintf ( '%o', fileperms ( $directory . $file ) ), -4 ) ."</li>\n";
	    	}
	    	//////////////////////////////////////////////////////////////////////////////

	    }

	}

	closedir ( $folder );
    } // removeDir

}

?>